#include <stdint.h>
#include <stdlib.h>

#include "elcore50-matrix-lib/mat_mul_with_dma_fl16.h"

void ref_convolution(const float * src, int batch, int srcH, int srcW, int srcC,
    int kernelY, int kernelX, int dilationY, int dilationX,
    int strideY, int strideX, int padY, int padX, int padH, int padW, int group,
    const float * weight, const float * bias, float * dst, int dstC);

void convolution_with_mat_mul(const float * src, int batch, int srcH, int srcW, int srcC,
    int kernelY, int kernelX, int dilationY, int dilationX,
    int strideY, int strideX, int padY, int padX, int padH, int padW, int group,
    const float * weight, const float * bias, float * dst, int dstC);

void convolution_with_mat_mul_fl16(uint16_t * src, int batch, int srcH, int srcW, int srcC,
    int kernelY, int kernelX, int dilationY, int dilationX,
    int strideY, int strideX, int padY, int padX, int padH, int padW, int group,
    uint16_t * weight, float * bias, uint16_t * dst, int dstH, int dstW, int dstC);

extern "C" void convolution_16x128_fl16(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                  int kernelY, int kernelX, int dilationY, int dilationX,
                                                  int strideY, int strideX, int padY, int padX, int padH, int padW, int group,
                                                  uint16_t * weight, float * bias, uint16_t * dst, int dstC);

extern "C" void convolution_16x128_fl16_aliquantM(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                  int kernelY, int kernelX, int dilationY, int dilationX,
                                                  int strideY, int strideX, int padY, int padX, int padH, int padW, int group,
                                                  uint16_t * weight, float * bias, uint16_t * dst, int dstC);
extern "C" void convolution_16x128_fl16_aliquantM_relu(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                       int kernelY, int kernelX, int dilationY, int dilationX,
                                                       int strideY, int strideX, int padY, int padX, int padH, int padW, int group,
                                                       uint16_t * weight, float * bias, uint16_t * dst, int dstC);
extern "C" void convolution_16x128_fl16_aliquantM_relu6(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                        int kernelY, int kernelX, int dilationY, int dilationX,
                                                        int strideY, int strideX, int padY, int padX, int padH, int padW, int group,
                                                        uint16_t * weight, float * bias, uint16_t * dst, int dstC);

extern "C" void convolution_16x128_fl16_aliquantM_bias(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                  int kernelY, int kernelX, int dilationY, int dilationX,
                                                  int strideY, int strideX, int padY, int padX, int padH, int padW, int group,
                                                  uint16_t * weight, float * bias, uint16_t * dst, int dstC);
extern "C" void convolution_16x128_fl16_aliquantM_relu_bias(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                       int kernelY, int kernelX, int dilationY, int dilationX,
                                                       int strideY, int strideX, int padY, int padX, int padH, int padW, int group,
                                                       uint16_t * weight, float * bias, uint16_t * dst, int dstC);
extern "C" void convolution_16x128_fl16_aliquantM_relu6_bias(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                        int kernelY, int kernelX, int dilationY, int dilationX,
                                                        int strideY, int strideX, int padY, int padX, int padH, int padW, int group,
                                                        uint16_t * weight, float * bias, uint16_t * dst, int dstC);

void convolution_with_dma(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                          int kernelY, int kernelX, int dilationY, int dilationX,
                          int strideY, int strideX, int dstH, int dstW, int group,
                          uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                          ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains, uint16_t* zeros);

void init_chain_dma_convolution_with_dma(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                         int kernelY, int kernelX, int dilationY, int dilationX,
                                         int strideY, int strideX, int dstH, int dstW, int group,
                                         int padX, int padW, int padY, int padH,
                                         uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                                         ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains, uint16_t* zeros);

void convolution_with_dma_1A2B2C(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                 int kernelY, int kernelX, int dilationY, int dilationX,
                                 int strideY, int strideX, int dstH, int dstW, int group,
                                 uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                                 ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains, uint16_t* zeros,
                                 Store_version st_ver, bool add_bias_flag);

void init_chain_dma_convolution_with_dma_1A2B2C(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                int kernelY, int kernelX, int dilationY, int dilationX,
                                                int strideY, int strideX, int dstH, int dstW, int group,
                                                int padX, int padW, int padY, int padH,
                                                uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                                                ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains, uint16_t* zeros, bool bias_flag);

void convolution_with_dma_2A2B2C(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                 int kernelY, int kernelX, int dilationY, int dilationX,
                                 int strideY, int strideX, int dstH, int dstW, int group,
                                 uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                                 ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains, uint16_t* zeros,
                                 Store_version st_ver, bool add_bias_flag);

void init_chain_dma_convolution_with_dma_2A2B2C(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                int kernelY, int kernelX, int dilationY, int dilationX,
                                                int strideY, int strideX, int dstH, int dstW, int group,
                                                int padX, int padW, int padY, int padH,
                                                uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                                                ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains, uint16_t* zeros, bool add_bias_flag);

void conv_size_selector_fl16(int batch, int srcH, int srcW, int srcC,
                             int kernelY, int kernelX,
                             int dstH, int dstW, int dstC,
                             int* ptr_tile_padX, int* ptr_tile_padY, int* ptr_tile_padW, int* ptr_tile_padH,
                             int* ptr_buf_srcH, int* ptr_buf_srcW,
                             int* ptr_buf_dstH, int* ptr_buf_dstW, int* ptr_buf_dstC,
                             int strideY, int strideX,
                             int count_buf_A, int count_buf_B, int count_buf_C, bool bias_flag, int local_mem_size);
